import time
import requests
from py_progress.progress import ProgressBar
import math
import threading

class Proxy:
    def __init__(self, proxi_link):
        self.proxy_link = proxi_link

    def ping(self):
        try:
            response = requests.get("http://httpbin.org/ip", proxies=self.proxy_link, timeout=5)
            return True, response.json()
        except Exception as e:
            return False, e

    def test_speed(self, obj):
        start_time = time.time()
        response = requests.get("https://file-examples.net/wp-content/uploads/2024/02/file-sample_100kB.doc", proxies=self.proxy_link, timeout=10)
        end_time = time.time()
        obj.set_speed(self, 1 / (end_time - start_time))
        return 1 / (end_time - start_time)
    
    def __repr__(self):
        return f"proxy: {self.proxy_link["http"]}"

class ProxiesManager:
    def __init__(self, proxies_array):
        self.proxies_array = proxies_array

    def set_speed(self, obj, time_used):
        self.proxies_speed_array.append([obj, time_used])


    def test_speed(self, progress_func_update=None):
        self.proxies_speed_array = []
        pb = ProgressBar(len(self.proxies_array))
        for el in self.proxies_array:
            threading.Thread(target=el.test_speed, args=(self,)).start()
            pb.progress_with_time("")
            if progress_func_update is not None:
                threading.Thread(target=progress_func_update, args=(math.ceil((pb.procent / pb.all)  * 100), "find proxies")).start()

        while len(self.proxies_array) > len(self.proxies_speed_array):
            time.sleep(0.5)

        return self.proxies_speed_array
    
    def get_best_proxies(self, max_proxies=15, progress_func_update=None):
        proxies_speed_array = self.test_speed(progress_func_update=progress_func_update)

        for i in range(len(proxies_speed_array)):
            for j in range(len(proxies_speed_array) - 1):
                if proxies_speed_array[j][1] > proxies_speed_array[j + 1][1]:
                    proxies_speed_array[j], proxies_speed_array[j + 1] = proxies_speed_array[j + 1], proxies_speed_array[j]

        proxies_speed_array.reverse()
        current_proxies = proxies_speed_array[:max_proxies]

        mean_speed = 0
        for el in current_proxies:
            mean_speed += el[1]

        mean_speed /= len(current_proxies)
        only_proxies = [el[0] for el in current_proxies]
        return current_proxies, only_proxies, mean_speed

from bs4 import BeautifulSoup
import requests

def load_proxies(base_url="https://free-proxy-list.net/ru/uk-proxy.html"):
    html_data = requests.get(base_url).text
    soup = BeautifulSoup(html_data, 'html.parser')

    textarea = soup.find('textarea', class_='form-control')

    all_proxyes_raw = textarea.text
    all_proxyes_text: str = all_proxyes_raw[75:-1]
    all_proxyes = all_proxyes_text.split("\n")
    proxies = [Proxy({"http": f"http://{url}"}) for url in all_proxyes]

    return proxies

if __name__ == "__main__":
    pm = ProxiesManager(load_proxies())
    print(pm.get_best_proxies(10))



import requests
from .. config import BASE_URL
from .. schema.profile_schema import AuthShema

class Profile:
    def __init__(self, sessiond_id=""):
        if sessiond_id != "":
            self.session_id = sessiond_id
        else:
            self.session_id = None

    @property
    def auth_cookies(self):
        return {"session_id": self.session_id}

    def regist(self, login, password):
        response = requests.post(f"{BASE_URL}/authorizate/regist", data={"login": login, "password": password})
        return response.json()
    
    def login_user(self, auth_data: AuthShema):
        response = requests.post(f"{BASE_URL}/authorizate/login", data={"login": auth_data.login, "password": auth_data.password})

        if response.json()["status"] == "success":
            self.session_id = response.json()["session_id"]
            return True, ""
        return False, response.json()["message"]
            

    def logout_user(self):
        response = requests.post(f"{BASE_URL}/authorizate/logout", cookies=self.auth_cookies)

    def get_version(self):
        try:
            response = requests.get(f"{BASE_URL}/authorizate/check_version")
            return True, response.json()["version"]

        except:
            return False, ""

    def get_tg_info(self):
        response = requests.get(f"{BASE_URL}/authorizate/bot_data", cookies=self.auth_cookies)
        return response.json()["data"]
    
    def is_auth(self):
        response = requests.get(f"{BASE_URL}/authorizate/check_auth", cookies=self.auth_cookies)
        if response.json()["status"] == "success":
            return True
        return False



import requests



class Bot:
    def __init__(self, token, chat_id):
        self.token = token
        self.chat_id = chat_id
        self.curr_proxy_index = 0
        self.session = requests.Session()

    def send_block(self, data, name, proxies):
        if self.curr_proxy_index >= len(proxies) - 1:
            self.curr_proxy_index = 0
        else:
            self.curr_proxy_index += 1

        if proxies is None:
            response = self.session.post(f"https://api.telegram.org/bot{self.token}/sendDocument", data={"chat_id": self.chat_id}, files={"document": (name, data)})

        response = self.session.post(f"https://api.telegram.org/bot{self.token}/sendDocument", proxies=proxies[self.curr_proxy_index].proxy_link, data={"chat_id": self.chat_id}, files={"document": (name, data)})


        res_data =  response.json()
        response.close()
        return res_data
    
    async def get_download_link_async(self, client, file_id):
        url = f"https://api.telegram.org/bot{self.token}/getFile"
        res = await client.get(url, params={"file_id": file_id})
        file_info = res.json()
        if file_info['ok'] == False:
            print(file_info, file_id)
            return "", False

        file_path = file_info["result"]["file_path"]
        download_url = f"https://api.telegram.org/file/bot{self.token}/{file_path}"

        return download_url, True
    
    def get_download_link(self, file_id):
        url = f"https://api.telegram.org/bot{self.token}/getFile"
        res = requests.get(url, params={"file_id": file_id})
        file_info = res.json()
        if file_info['ok'] == False:
            print(file_info, file_id)
            return "", False

        file_path = file_info["result"]["file_path"]
        download_url = f"https://api.telegram.org/file/bot{self.token}/{file_path}"

        return download_url, True
    



from abc import ABC
from .. api_workers.tg_handle import Bot
import time
import math
from py_progress.progress import ProgressBar
import requests
from .. utils.proxies_getter import Proxy
import threading
from .. config import BASE_URL



class Downloader(ABC):
    def __init__(self, bot, proxies):
        self.bot: Bot = bot
        self.proxies: list[Proxy] = proxies
        self.session = requests.Session()


    def download_block(self, url, index, proxy_index):

        self.q_count += 1
        durl, status_link = self.bot.get_download_link(url)

        try:
            response = self.session.get(durl, proxies=self.proxies[proxy_index].proxy_link, timeout=None)

            self.current_blocks.append([index, response.content])
            self.q_count -= 1
            self.pb.progress_with_time("")
            if self.progress_fun is not None:
                self.progress_fun(math.ceil((self.pb.procent / self.pb.all)  * 100), "download")

        except:
            self.proxies.pop(proxy_index)
            self.add_to_curr_proxy()
            print("del")
            self.download_block(url, index, self.curr_proxy_index)


    def sort_blocks(self, blocks):
        sort_blocks = blocks.copy()
        for i in range(len(blocks)):
            for j in range(len(blocks) - 1):
                if sort_blocks[j][0] > sort_blocks[j + 1][0]:
                    sort_blocks[j], sort_blocks[j + 1] = sort_blocks[j + 1], sort_blocks[j]
        return sort_blocks
    
    def wating_blocks(self):
        while True:
            if len(self.current_blocks) <= 1:
                break

            time.sleep(1)

    def queue_wating(self, max_blocks):
        while True:
            if self.q_count <= max_blocks:
                break

            time.sleep(1)

    def final_pool_blocks_with_q(self, file_path):
        
        all_sorted_blocks = self.sort_blocks(self.current_blocks)
        check_line_sorted = []
        if len(all_sorted_blocks) > 0:

            for i, el in enumerate(all_sorted_blocks):
                if i > 0:
                    if all_sorted_blocks[i-1][0] + 1 != el[0]:
                        break

                check_line_sorted.append(el)
            

            if check_line_sorted[0][0] == self.last_index + 1:
                for i, el in enumerate(check_line_sorted):
                    
                    with open(file_path, "ab") as f:
                        f.write(el[1])


                self.last_index = check_line_sorted[-1][0]
                self.current_blocks = all_sorted_blocks[len(check_line_sorted):]

    def add_to_curr_proxy(self):
        if self.curr_proxy_index >= len(self.proxies) - 1:
            self.curr_proxy_index = 0
        else:
            self.curr_proxy_index += 1

    def download(self, files_id, file_path, max_blocks_pool=10, progress_fun=None, full_check_in=50):
        self.pb = ProgressBar(math.ceil(len(files_id)))
       
        self.curr_proxy_index = 0

        self.current_blocks = []
        self.q_count = 0
        self.last_index = -1
        self.progress_fun = progress_fun

        for i in range(len(files_id)):
            #choese proxy
            # curr_proxy = self.proxies[curr_proxy_index]
            

            #start_block
            down = threading.Thread(target=self.download_block, args=(files_id[i], i, self.curr_proxy_index))
            down.start()

            self.add_to_curr_proxy()

            self.queue_wating(max_blocks_pool)
            self.final_pool_blocks_with_q(file_path)
            if i % full_check_in == 0:
                while self.q_count > 0:
                    time.sleep(1)


        while self.q_count > 0:
            time.sleep(1)
        self.final_pool_blocks_with_q(file_path)



   







from .api_workers.auth import Profile
from .files_workers.downloader import Downloader
from .files_workers.uploader import Loader
from .utils.proxies_getter import load_proxies, ProxiesManager
from .api_workers.auth import Profile
from .api_workers.file_manager_api import get_global_file_info, get_all_batch_files, get_all_global_files, delete_file_api, get_last_batch_id, get_unclose_upload_sessions
from .api_workers.tg_handle import Bot
import threading
from .schema.profile_schema import AuthShema


class FullManager:
    def __init__(self, TOKEN, CHAT_ID, profile: Profile, type_of_init: int = 0):
        self.bot = Bot(TOKEN, CHAT_ID)
        self.profile = profile
        self.want_upload_queue = []

        
        self.best_proxies = load_proxies()[:15]

        if type_of_init == 0:
            threading.Thread(target=self.get_proxies, args=(30,)).start()
        elif type_of_init == 1:
            self.get_proxies(30)

    def get_proxies(self, max_proxies):
        _, self.best_proxies, mean_speed = ProxiesManager(load_proxies()[:100]).get_best_proxies(max_proxies=max_proxies)
        print(f"\nProxies mean speed: {mean_speed:.2f} mb/s")


    def want_upload(self, *args, **kwargs):
        curr_loader, block_size, file_size = self.prepere_to_queue(*args, **kwargs)
        
        self.want_upload_queue.append(lambda: self.upload(curr_loader, *args, **kwargs))
        self.run_queue(True)

        return {
            "upload_session_id": curr_loader.upload_session_id,
            "block_size": block_size,
            "file_size": file_size
        }

    def run_queue(self, isFromClient: bool):
        if isFromClient:
            if len(self.want_upload_queue) == 1:
                pass
            else:
                return
        else:
            if len(self.want_upload_queue) > 0:
                pass
            else:
                return
            
        threading.Thread(target=self.want_upload_queue[0]).start()

    def prepere_to_queue(self, file_path, block_size=10, max_pool_blocks=5, custom_bar_func=None, upload_session=None, start_index=0):
        with open(file_path, "rb") as file:
            l = Loader(file, self.bot, self.profile, proxies=self.best_proxies)
            if upload_session is None:
                l.init_global_file(file_path=file_path)
            else:
                l.upload_session_id = upload_session

            file_size = l.file_size

        return l, block_size, file_size

    def download(self, global_file_id, output_folder: str, max_proxies=15, max_block_pool=10, custom_bar_func=None):
        file_info = get_global_file_info(profile=self.profile, global_file_id=global_file_id)
        files_data = get_all_batch_files(self.profile, global_file_id=global_file_id)["files_data"]

        all_urls = [el["file_tg_hash"] for el in files_data]

        d = Downloader(self.bot, proxies=self.best_proxies)
        d.download(all_urls, f"{output_folder}\\{file_info["file_name"]}", max_blocks_pool=max_block_pool, progress_fun=custom_bar_func)

    def upload(self, l: Loader, file_path, max_proxies=15, block_size=10, max_pool_blocks=5, custom_bar_func=None, start_index=0, upload_session=None):

        with open(file_path, "rb") as file:
            l.file = file
            l.split_info(block_size * 1024 * 1024, max_pool_blocks, progress_fun=custom_bar_func, start_index=start_index)

        self.want_upload_queue.pop(0)
        self.run_queue(False)


        return l.upload_session_id
    

class TgSaver:
    def __init__(self, auth_data: AuthShema):
        self.profile = Profile("")
        status, message = self.profile.login_user(auth_data)

        if status == False:
            raise Exception(f"incorrect auth {message}")
        
        bot_data = self.profile.get_tg_info()
        self.TOKEN, self.CHAT_ID = bot_data["BOT_TOKEN"], bot_data["CHAT_ID"]


        self.manager = FullManager(self.TOKEN, self.CHAT_ID, self.profile, type_of_init=1)
        
    def upload_file(self, path: str):
        base_file_info = self.manager.want_upload(path)
        return base_file_info
    
    def download_file(self, output_path: str, global_file_id: int):
        self.manager.download(global_file_id, output_path)

    def get_all_global_files(self):
        return get_all_global_files(self.profile)
    

if __name__ == "__main__":
    tg_saver = TgSaver(AuthShema(login="test", password="test"))
    print(tg_saver.get_all_global_files())
    tg_saver.upload_file(r"C:\Users\huina\Videos\2025-07-11 14-01-30.mp4")
    # tg_saver.download_file(r"C:\Users\huina\Python Projects\Test Projects\TEST_LIB\test_download", 25)



import requests
from .auth import Profile
from .. config import BASE_URL



def get_unclose_upload_sessions(profile: Profile):
    response = requests.get(f"{BASE_URL}/files_handler/unclose_upload_session", cookies=profile.auth_cookies)
    

    return response.json()["unclose_upload_sessions_id"]

def get_last_batch_id(profile: Profile, upload_session_id):
    response = requests.get(f"{BASE_URL}/files_handler/{upload_session_id}/last_batch_id", cookies=profile.auth_cookies)

    return response.json()["last_batch_id"]

def delete_file_api(profile: Profile, global_file_id):
    response = requests.post(f"{BASE_URL}/files_handler/{global_file_id}/delete", cookies=profile.auth_cookies)

    return response.json()

def get_all_global_files(profile: Profile):
    response = requests.get(f"{BASE_URL}/files_handler/get_all_global_files", cookies=profile.auth_cookies)

    return response.json()["files_data"]

def get_all_batch_files(profile: Profile, global_file_id):
    response = requests.get(f"{BASE_URL}/files_handler/{global_file_id}/batches", cookies=profile.auth_cookies)

    return response.json()

def get_global_file_info(profile: Profile, global_file_id):
    response = requests.get(f"{BASE_URL}/files_handler/{global_file_id}/info", cookies=profile.auth_cookies)

    return response.json()["files_data"]


import threading
from abc import ABC, abstractmethod
import os
import hashlib
from .. api_workers.tg_handle import Bot
import io
import time
from py_progress.progress import ProgressBar
import requests
from .. api_workers.auth import Profile
from .. utils.proxies_getter import load_proxies, Proxy
import math
from .. config import BASE_URL




def get_random_hash(random_param=32):
    random_bytes = os.urandom(random_param)
    hash_value = hashlib.sha256(random_bytes).hexdigest()

    return hash_value



class Loader(ABC):
    def __init__(self, file, bot, profile: Profile, proxies):
        self.file = file
        self.bot: Bot = bot
        self.count_of_file = 0
        self.profile = profile
        self.proxies = proxies
        self.session = requests.Session()

    def init_global_file(self, file_path=None):
        res = self.session.post(f"{BASE_URL}/files_handler/add_global_file", cookies=self.profile.auth_cookies, data={
            "file_size": self.file_size,
            "file_name": self.file.name.split("\\")[-1],
            "file_path": file_path
        })
        res_data = res.json()
        self.upload_session_id = res_data["upload_session_id"]

    @property
    def file_size(self):
        pos = self.file.tell()
        self.file.seek(0, 2)  
        size = self.file.tell()
        self.file.seek(pos)

        return size


    def send_block(self, file_index, block, file_hash):
        self.req_in_q += 1
        while True:
            response_data = self.bot.send_block(io.BytesIO(block), f"{file_hash}", proxies=self.proxies)
            if response_data["ok"] == True:
                break
            
            print(f"this proxy doestn work: {self.bot.curr_proxy_index}")
            self.proxies.pop(self.bot.curr_proxy_index)
            self.bot.curr_proxy_index -= 1

        res_data = self.session.post(f"{BASE_URL}/files_handler/{self.upload_session_id}/add", cookies=self.profile.auth_cookies, data={
            "file_tg_hash": response_data["result"]["document"]["file_id"],
            "file_index": file_index,
            "file_size": response_data["result"]["document"]["file_size"],
        })
        self.count_of_file += 1
        self.req_in_q -= 1

        self.current_pb.progress("")
        if self.progress_fun is not None:
            self.progress_fun(self.upload_session_id, math.ceil((self.current_pb.procent / self.current_pb.all)  * 100))


    def reader(self, chunk_size):
        buffer = b''
        while chunk := self.file.read(chunk_size):
            is_last = len(chunk) < chunk_size
            chunk = buffer + chunk

            if is_last == False:

                last_index = chunk.rfind(b"\n")
                if last_index == -1:
                    buffer = b''
                else:
                    buffer = chunk[last_index + 1:]
                    chunk = chunk[:last_index + 1]
                
                
                yield chunk
            else:
                yield chunk

    def check_len(self, target_len):
        while True:
            if target_len <= self.count_of_file:
                break
            time.sleep(1)

    def check_len_queue(self, max_q):
        while True:
            if self.req_in_q <= max_q:
                break
            time.sleep(1)

    def split_info(self, block_size=10 * 1024 * 1024, send_controller=10, progress_fun=None, start_index=0, full_check_in=50):
        response_data_len = 0

        start_time = time.time()
        blocks_count = math.ceil(math.ceil(self.file_size / (block_size)))
        self.req_in_q = 0

        self.progress_fun = progress_fun
        self.current_pb = ProgressBar(blocks_count)
        self.current_pb.procent += start_index

        for i, el in enumerate(self.reader(block_size)):
            if i >= start_index:
                file_hash = get_random_hash()
                file_hash += ".bin"

                send = threading.Thread(target=self.send_block, args=(i, el, file_hash))       
                send.start()
                response_data_len += 1

                self.check_len_queue(send_controller)

                if i % full_check_in == 0:
                    self.check_len(response_data_len)




                
                    



        self.check_len(response_data_len)

        res_data = requests.post(f"{BASE_URL}/files_handler/{self.upload_session_id}/close", cookies=self.profile.auth_cookies)
        print("END")




if __name__ == "__main__":

    TOKEN = "8392027187:AAEglTR9vtwC_yYFtNZUE8zuQABBGGCJdXE"
    CHAT_ID = "1228194994"

    start_time = time.time() 
    profile = Profile("test", "test")

    proxies = load_proxies()
    with open(r"C:\Users\huina\Downloads\video_2025-08-08_13-18-11.mp4", "rb") as file:

        l = Loader(file, Bot(TOKEN, CHAT_ID), profile, proxies=proxies[:15])
        l.init_global_file()
        response_data = l.split_info(10 * 1024 * 1024, 5)

    print(l.global_file_id)
    endload_time = time.time() 

    print(f"\nupload time = {(endload_time - start_time):.2f}")
